@@ -17,31 +17,36 @@
 #  NOTE: 'unityForest' is a fork of the popular R package 'ranger', written by Marvin N. Wright.
 #  Most R and C++ code is identical with that of 'ranger'. The package 'unityForest'
 #  was written by taking the original 'ranger' code and making any
-#  changes necessary to implement diversity forests.
+#  changes necessary to implement unity forests.
 #
 # -------------------------------------------------------------------------------
 
 ##' Prediction with new data and a saved forest from \code{\link{unityfor}}.
-##' 
-##' This package is a fork of the R package 'ranger' that implements random forests using an
-##' efficient C++ implementation. More precisely, 'unityForest' was written by modifying
-##' the code of 'ranger', version 0.11.0. 
+##'
+##' For \code{type = 'response'} (the default), the predicted probabilities (probability estimation), predicted classes (classification), or predicted numeric values (continuous outcomes) are returned. 
+##' For \code{type = 'terminalNodes'}, the IDs of the terminal node in each tree for each observation in the given dataset are returned.
 ##'
 ##' @title Unity Forest prediction
 ##' @param object \code{unityfor} object.
 ##' @param data New test data of class \code{data.frame}.
-##' @param predict.all Return individual predictions for each tree instead of aggregated predictions for all trees. Return a matrix (sample x tree) for classification and a 3d array for probability estimation (sample x class x tree).
+##' @param predict.all Return individual predictions for each tree instead of aggregated predictions for all trees. Return a matrix (sample x tree) for classification and continuous outcomes, and a 3d array for probability estimation (sample x class x tree).
 ##' @param num.trees Number of trees used for prediction. The first \code{num.trees} in the forest are used.
-##' @param type Type of prediction. One of 'response', 'se', 'terminalNodes', 'quantiles' with default 'response'. See below for details.
-##' @param num.threads Number of threads. Default is number of CPUs available.
+##' @param type Type of prediction. Either 'response' (default) or 'terminalNodes'. See below for details.
+##' @param num.threads Number of threads to use.
+##' The default is to use at most 2 threads (and at most the number of available CPU cores).
+##' This conservative default avoids unintentionally using many cores on shared computing resources
+##' (e.g., CI systems, servers, or HPC login/compute nodes).
+##'
+##' For typical use on a personal computer, setting \code{num.threads = 0} is strongly recommended,
+##' as it uses all available CPU cores, which typically substantially reduces runtime.
 ##' @param verbose Verbose output on or off.
 ##' @param ... further arguments passed to or from other methods.
 ##' @return Object of class \code{unityfor.prediction} with elements
 ##'   \tabular{ll}{
-##'       \code{predictions}    \tab Predicted classes/probabilities/values (only for classification and regression)  \cr
+##'       \code{predictions}    \tab Predicted classes/probabilities/values/terminal nodes  \cr
 ##'       \code{num.trees}   \tab Number of trees. \cr
 ##'       \code{num.independent.variables} \tab Number of independent variables. \cr
-##'       \code{treetype}    \tab Type of forest/tree. Classification or regression (the latter is not implemented yet). \cr
+##'       \code{treetype}    \tab Type of forest/tree. Categorical or continuous outcome. \cr
 ##'       \code{num.samples}     \tab Number of samples.
 ##'   }
 ##' @references
@@ -50,7 +55,7 @@
 ##'   \item Wright, M. N., Ziegler, A. (2017). ranger: A fast Implementation of Random Forests for High Dimensional Data in C++ and R. Journal of Statistical Software 77:1-17, <\doi{10.18637/jss.v077.i01}>.
 ##'   }
 ##' @seealso \code{\link{unityfor}}
-##' @author Marvin N. Wright
+##' @author Marvin N. Wright, Roman Hornung
 ##' @export
 predict.unityfor <- function(object, data = NULL, predict.all = FALSE,
                            num.trees = object$num.trees,
@@ -193,12 +198,28 @@ predict.unityfor.forest <- function(object, data, predict.all = FALSE,
   }
 
   ## Num threads
-  ## Default 0 -> detect from system in C++.
+  ## Default: use at most 2 threads. Set num.threads = 0 to use all available cores.
   if (is.null(num.threads)) {
-    num.threads = 0
+    num.threads <- 2L
   } else if (!is.numeric(num.threads) || num.threads < 0) {
     stop("Error: Invalid value for num.threads")
   }
+  
+  ## Respect core limits during R CMD check / CRAN-like checks
+  ## (only if running under package checks)
+  if (nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_", unset = ""))) {
+    limit_cores <- Sys.getenv("_R_CHECK_LIMIT_CORES_", unset = "")
+    if (nzchar(limit_cores)) {
+      lim <- suppressWarnings(as.integer(limit_cores))
+      if (!is.na(lim) && lim > 0) {
+        if (num.threads == 0) {
+          num.threads <- lim
+        } else {
+          num.threads <- min(num.threads, lim)
+        }
+      }
+    }
+  }
 
   ## Seed
   if (is.null(seed)) {
@@ -247,10 +268,7 @@ predict.unityfor.forest <- function(object, data, predict.all = FALSE,
   max.depth <- 0
   inbag <- list(c(0,0))
   use.inbag <- FALSE
-  nsplits <- 0 ## asdf
-  proptry <- 0 ## asdf
-  eim.mode <- 0
-  
+
   ## Use sparse matrix
   if ("dgCMatrix" %in% class(data.final)) {
     sparse.data <- data.final
@@ -274,8 +292,8 @@ predict.unityfor.forest <- function(object, data, predict.all = FALSE,
                       case.weights, use.case.weights, class.weights, 
                       predict.all, keep.inbag, sample.fraction, alpha, minprop, holdout, 
                       prediction.type, num.random.splits, sparse.data, use.sparse.data,
-                      order.snps, oob.error, max.depth, max_depth_root=3, num_cand_trees=1000, inbag, use.inbag, nsplits, npairs=0, proptry, prop_var_root=0, divfortype=4, 
-					  promispairs=list(0,0), eim_mode=0, metricind=numeric(0), prop_best_splits=0, repr.tree.mode, repr.var.names) ## asdf
+                      order.snps, oob.error, max.depth, max_depth_root=3, num_cand_trees=1000, inbag, use.inbag, prop_var_root=0, 
+					  prop_best_splits=0, repr.tree.mode, repr.var.names) ## asdf
 
   if (length(result) == 0) {
     stop("User interrupt or internal error.")
@@ -371,7 +389,7 @@ predict.unityfor.forest <- function(object, data, predict.all = FALSE,
       jab <- pmax(jack - bias, 0)
       result$se <- sqrt(jab)
     } else if (se.method == "infjack") {
-	  stop("Error: 'se.method == infjack' currently not supported for diversity forests.")
+	  stop("Error: 'se.method == infjack' currently not supported for unity forests.")
       #if (forest$treetype == "Regression") {
       #  infjack <- rInfJack(pred = result$predictions, inbag = inbag.counts, used.trees = 1:num.trees)
       #  result$se <- sqrt(infjack$var.hat)
