@@ -17,7 +17,7 @@
 #  NOTE: 'unityForest' is a fork of the popular R package 'ranger', written by Marvin N. Wright.
 #  Most R and C++ code is identical with that of 'ranger'. The package 'unityForest'
 #  was written by taking the original 'ranger' code and making any
-#  changes necessary to implement diversity forests.
+#  changes necessary to implement unity forests.
 #
 # -------------------------------------------------------------------------------
 
@@ -53,7 +53,9 @@
 ##' Note that additional splits on the variable of interest may occur within shaded regions and can still be relevant. However, these splits do not represent the conditions under which the variable attains its strongest effects.
 ##' 
 ##' \strong{In-bag data for top-scoring split visualizations.}
-##' The boxplots and density plots illustrating the discriminatory power of the top-scoring splits are computed exclusively based on the in-bag observations of the corresponding trees. This is consistent with the construction of the CRTRs themselves, which are derived from in-bag data only.
+##' The boxplots/density plots/scatter plots illustrating the discriminatory power of the top-scoring splits are computed exclusively based on the in-bag observations of the corresponding trees. This is consistent with the construction of the CRTRs themselves, which are derived from in-bag data only.
+##' 
+##' NOTE: The empirical evaluation of the unity forest framework (including the unity forest algorithm, the unity VIM, and covariate-representative tree roots) in Hornung & Hapfelmeier (2026) focused on categorical outcomes. Its performance for continuous outcomes has not yet been systematically investigated. Results for continuous outcomes should therefore be interpreted with appropriate caution.
 ##' 
 ##' @title Select and visualize covariate-representative tree roots (CRTRs)
 ##' 
@@ -61,30 +63,51 @@
 ##' @param vars This is an optional vector of variable names, for which CRTRs should be obtained
 ##' @param numvars The number of the variables with the largest unity VIM values for which CRTRs should be obtained.
 ##' @param indvars The indices of the variables with the largest unity VIM values for which CRTRs should be obtained. For example, if \code{indvars = c(1, 3)}, the CRTRs for the variables with the largest and third-largest unity VIM values are obtained.
-##' @param num.threads Number of threads. Default is number of CPUs available.
+##' @param num.threads Number of threads to use.
+##' The default is to use at most 2 threads (and at most the number of available CPU cores).
+##' This conservative default avoids unintentionally using many cores on shared computing resources
+##' (e.g., CI systems, servers, or HPC login/compute nodes).
+##'
+##' For typical use on a personal computer, setting \code{num.threads = 0} is strongly recommended,
+##' as it uses all available CPU cores, which typically substantially reduces runtime.
 ##' @param plotit Whether or not the CRTRs should be plotted or merely returned (invisibly). Default is \code{TRUE}.
 ##' @param highlight_relevant Whether or not the nodes not containing the top-scoring splits for the variables of interest or their ancestor nodes should be shaded out. Default is \code{TRUE}. See the 'Details' section below for explanation.
 ##' @param box_plots Whether boxplots should be used to show the outcome class-specific distributions of the variables values in the nodes with top-scoring splits (see 'Details' section for explanation). For classification only. Default is \code{TRUE}.
 ##' @param density_plots Whether kernel density plots should be used to show the outcome class-specific distributions of the variable values in the nodes with top-scoring splits (see 'Details' section for explanation). For classification only. Default is \code{TRUE}.
-##' @param add_split_line Whether in the boxplots and/or density plots a line at the split point of the corresponding node should be drawn. Default is \code{TRUE}.
+##' @param scatter_plots Whether scatter plots should be used to investigate the relationship between the variables of interest and the outcome in the nodes with top-scoring splits (see 'Details' section for explanation). For continuous outcomes only. Default is \code{TRUE}.
+##' @param add_split_line Whether in the boxplots/density plots/scatter plots a line at the split point of the corresponding node should be drawn. Default is \code{TRUE}.
 ##' @param verbose Verbose output on or off. Default is \code{TRUE}.
 ##' @return Object of class \code{unityfor.reprTrees} with elements
-##'   \item{\code{rules}}{List. Ing-bag statistics on the outcome at each node in the CRTRs. For classification, this provides the class frequencies and the numbers of observations representing each class.}
+##'   \item{\code{rules}}{List. In-bag statistics on the outcome at each node in the CRTRs. For classification, this provides the class frequencies and the numbers of observations representing each class.}
 ##'   \item{\code{plots}}{List. Generated ggplot2 plots.}
 ##'   \item{\code{var.names}}{Labels of the variables for which CRTRs were selected.} 
-##'   \item{\code{var.names.all}}{Names of all independent variables in the dataset.}
+##'   \item{\code{independent.variable.names}}{Names of all independent variables in the dataset.}
 ##'   \item{\code{num.independent.variables}}{Number of independent variables in the dataset.}
 ##'   \item{\code{num.samples}}{Number of observations in the dataset.}
 ##'   \item{\code{treetype}}{Tree type.}
 ##'   \item{\code{forest}}{Sub-forest that contains only the CRTRs.}
 ##' @examples
 ##' \donttest{
-##'
+##' 
+##' ## IMPORTANT NOTE on parallelization:
+##' ## The default uses at most 2 threads (num.threads) to avoid unintentionally
+##' ## using many cores on shared systems.
+##' ## However, for typical runs on a personal computer, set num.threads = 0 to 
+##' ## use all available CPU cores; this is strongly recommended and can 
+##' ## substantially reduce runtime.
+##' ## Note: num.threads = 1 is used in the examples to avoid parallel
+##' ## execution during package checks.
+##' 
+##' 
 ##' ## Load package:
 ##' 
 ##' library("unityForest")
 ##' 
 ##' 
+##' 
+##' ## Categorical outcome:
+##' #######################
+##' 
 ##' ## Set seed to make results reproducible:
 ##' 
 ##' set.seed(1234)
@@ -98,7 +121,7 @@
 ##' ## Construct unity forest and calculate unity VIM values:
 ##' 
 ##' model <- unityfor(dependent.variable.name = "C", data = wine,
-##'                   importance = "unity", num.trees = 2000)
+##'                   importance = "unity", num.trees = 2000, num.threads = 1)
 ##' 
 ##' # NOTE: num.trees = 2000 (in the above) would be too small for practical 
 ##' # purposes. This quite small number of trees was simply used to keep the
@@ -109,13 +132,15 @@
 ##' ## Visualize the CRTRs for the five variables with the largest unity VIM
 ##' ## values:
 ##' 
-##' reprTrees(model, box_plots = FALSE, density_plots = FALSE)
+##' reprTrees(model, box_plots = FALSE, density_plots = FALSE,
+##'           num.threads = 1)
 ##' 
 ##' 
 ##' ## Visualize the CRTRs for the variables with the largest and third-largest 
 ##' ## unity VIM values:
 ##' 
-##' reprTrees(model, indvars = c(2, 3), box_plots = FALSE, density_plots = FALSE)
+##' reprTrees(model, indvars = c(2, 3), box_plots = FALSE, density_plots = FALSE,
+##'           num.threads = 1)
 ##' 
 ##' 
 ##' ## Visualize the CRTRs for the variables with the largest and third-largest 
@@ -123,7 +148,8 @@
 ##' ## outcome class-specific distributions of the variables values in the 
 ##' ## nodes with top-scoring splits:
 ##' 
-##' reprTrees(model, indvars = c(2, 3), box_plots = FALSE, density_plots = TRUE)
+##' reprTrees(model, indvars = c(2, 3), box_plots = FALSE, density_plots = TRUE,
+##'           num.threads = 1)
 ##' 
 ##' 
 ##' ## Visualize the CRTRs for the variables with the largest and third-largest 
@@ -132,7 +158,8 @@
 ##' ## in the top-scoring splits; the split points are not indicated in these
 ##' ## plots:
 
-##' ps <- reprTrees(model, indvars = c(2, 3), add_split_line = FALSE)
+##' ps <- reprTrees(model, indvars = c(2, 3), add_split_line = FALSE, 
+##'                 num.threads = 1)
 ##' 
 ##' 
 ##' ## Save one of the CRTRs with the corresponding density plot:
@@ -157,13 +184,70 @@
 ##'     legend.justification = c(1, 1)
 ##'   )
 ##' 
+##' 
+##' 
+##' 
+##' ## Continuous outcome:
+##' ######################
+##' 
+##' 
+##' ## Set seed to make results reproducible:
+##' 
+##' set.seed(1234)
+##' 
+##' 
+##' ## Load stock dataset:
+##' 
+##' data(stock)
+##' 
+##' 
+##' ## Construct unity forest and calculate unity VIM values:
+##' 
+##' model <- unityfor(dependent.variable.name = "company10", data = stock,
+##'                   importance = "unity", num.trees = 2000, num.threads = 1)
+##' 
+##' # NOTE: num.trees = 2000 (in the above) would be too small for practical 
+##' # purposes. This quite small number of trees was simply used to keep the
+##' # runtime of the example short.
+##' # The default number of trees is num.trees = 20000.
+##' 
+##' 
+##' ## Visualize the CRTRs for the variables "company1" und "company7", where
+##' ## scatter plots are shown to  visualize the effect of the variables in the 
+##' ## top-scoring splits:
+##' 
+##' ps <- reprTrees(model, vars = c("company1", "company7"), num.threads = 1)
+##' 
+##' 
+##' ## Visualize the CRTRs for the variables "company1" and "company7" without 
+##' ## scatter plots:
+##' 
+##' reprTrees(model, vars = c("company1", "company7"), scatter_plots = FALSE,
+##'           num.threads = 1)
+##' 
+##' 
+##' 
+##' # As also shown above, the plots can be manipulated with the usual 
+##' # ggplot2 syntax, e.g.:
+##' 
+##' library("ggplot2")
+##' 
+##' p <- ps$plots[[1]]$scatter_plot + labs(x = "Stock price of company 1",
+##'                                        y = "Stock price of company 10") + 
+##'   ggtitle("Marginal influence of the stock price of company 1")
+##' p
+##' 
+##' p$layers[[1]]$aes_params$shape <- 1
+##' p$layers[[1]]$aes_params$colour <- "red"
+##' p
+##' 
 ##' }
-##'
+##' 
 ##' @author Roman Hornung
 ##' @references
 ##' \itemize{
 ##'   \item Hornung, R., Hapfelmeier, A. (2026). Unity Forests: Improving Interaction Modelling and Interpretability in Random Forests. arXiv:2601.07003, <\doi{10.48550/arXiv.2601.07003}>.
-##'   \item Laabs, B.-H., Westenberger, A., & K\"onig, I. R. (2024). Identification of representative trees in random forests based on a new tree-based distance measure. Advances in Data Analysis and Classification 18(2):363-380, <\doi{10.1007/s11634-023-00537-7}>.
+##'   \item Laabs, B.-H., Westenberger, A., & König, I. R. (2024). Identification of representative trees in random forests based on a new tree-based distance measure. Advances in Data Analysis and Classification 18(2):363-380, <\doi{10.1007/s11634-023-00537-7}>.
 ##'   }
 ##' @seealso \code{\link{unityfor}}
 ##' @encoding UTF-8
@@ -174,7 +258,7 @@
 ##' @import utils
 ##' @export
 reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads = NULL, plotit=TRUE,
-                         highlight_relevant = TRUE, box_plots = TRUE, density_plots = TRUE, add_split_line = TRUE, verbose = TRUE) {
+                         highlight_relevant = TRUE, box_plots = TRUE, density_plots = TRUE, scatter_plots = TRUE, add_split_line = TRUE, verbose = TRUE) {
   
   predict.all <- FALSE
   num.trees <- object$num.trees
@@ -259,13 +343,29 @@ reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads =
   }
   
   ## Num threads
-  ## Default 0 -> detect from system in C++.
+  ## Default: use at most 2 threads. Set num.threads = 0 to use all available cores.
   if (is.null(num.threads)) {
-    num.threads = 0
+    num.threads <- 2L
   } else if (!is.numeric(num.threads) || num.threads < 0) {
     stop("Error: Invalid value for num.threads")
   }
   
+  ## Respect core limits during R CMD check / CRAN-like checks
+  ## (only if running under package checks)
+  if (nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_", unset = ""))) {
+    limit_cores <- Sys.getenv("_R_CHECK_LIMIT_CORES_", unset = "")
+    if (nzchar(limit_cores)) {
+      lim <- suppressWarnings(as.integer(limit_cores))
+      if (!is.na(lim) && lim > 0) {
+        if (num.threads == 0) {
+          num.threads <- lim
+        } else {
+          num.threads <- min(num.threads, lim)
+        }
+      }
+    }
+  }
+  
   ## Seed
   if (is.null(seed)) {
     seed <- runif(1 , 0, .Machine$integer.max)
@@ -281,6 +381,11 @@ reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads =
     stop("Error: Unknown tree type.")
   }
   
+  if (treetype==3) {
+    box_plots <- FALSE
+	density_plots <- FALSE
+  }
+  
   ## Defaults for variables not needed
   dependent.variable.name <- ""
   mtry <- 0
@@ -313,9 +418,6 @@ reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads =
   max.depth <- 0
   inbag <- list(c(0,0))
   use.inbag <- FALSE
-  nsplits <- 0 ## asdf
-  proptry <- 0 ## asdf
-  eim.mode <- 0
   prediction.type <- 1
   prop.best.splits <- object$prop.best.splits
   
@@ -341,8 +443,8 @@ reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads =
                       case.weights, use.case.weights, class.weights, 
                       predict.all, keep.inbag, sample.fraction, alpha, minprop, holdout, 
                       prediction.type, num.random.splits, sparse.data, use.sparse.data,
-                      order.snps, oob.error, max.depth, max_depth_root=3, num_cand_trees=1000, inbag, use.inbag, nsplits, npairs=0, proptry, prop_var_root=0, divfortype=4, 
-                      promispairs=list(0,0), eim_mode=0, metricind=numeric(0), prop.best.splits,
+                      order.snps, oob.error, max.depth, max_depth_root=3, num_cand_trees=1000, inbag, use.inbag, prop_var_root=0, 
+                      prop.best.splits,
                       repr.tree.mode, repr.var.names)
   
   
@@ -356,7 +458,7 @@ reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads =
   plots_and_rules <- list()
   for(i in seq(along=repr.var.names)) {
     
-    plots_and_rules[[i]] <- .plot_representative_tree(i, result$forest, data, names(data)[forest$dependent.varID+1], repr.var.names, forest$independent.variable.names, plotit, highlight_relevant, box_plots, density_plots, add_split_line)
+    plots_and_rules[[i]] <- .plot_representative_tree(i, result$forest, data, names(data)[forest$dependent.varID+1], repr.var.names, forest$independent.variable.names, plotit, highlight_relevant, box_plots, density_plots, scatter_plots, add_split_line, treetype)
     
     if(plotit) {
       for (j in seq(along=plots_and_rules[[i]]$ps)) {
@@ -384,14 +486,14 @@ reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads =
   result$num.samples <- nrow(data.final)
   result$var.names <- repr.var.names
   result$treetype <- forest$treetype
-  result$var.names.all <- forest$independent.variable.names
+  result$independent.variable.names <- forest$independent.variable.names
   result$predictions <- result$num.trees <- NULL
   
   
   # Reorder the result list, so that they items in the list have a meaningful
   # ordering:
   
-  res_names_all <- c("rules", "plots", "var.names",  "var.names.all", "num.independent.variables", "num.samples", "treetype", "forest")
+  res_names_all <- c("rules", "plots", "var.names",  "independent.variable.names", "num.independent.variables", "num.samples", "treetype", "forest")
   res_names <- names(result)
   
   ind_cl <- which(res_names %in% res_names_all)
@@ -417,12 +519,15 @@ reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads =
 
 
 # Function for plotting one of the representative trees:
-.plot_representative_tree <- function(treeind, forest, data, dep.var.name, var.names, var.names.all, plotit=TRUE, highlight_relevant=TRUE, box_plots=TRUE, density_plots=TRUE, add_split_line=TRUE) {
+.plot_representative_tree <- function(treeind, forest, data, dep.var.name, var.names, independent.variable.names, plotit=TRUE, highlight_relevant=TRUE, box_plots=TRUE, density_plots=TRUE, scatter_plots=TRUE, add_split_line=TRUE, treetype) {
   
   # Subset the data to only contain the in-bag observations of the tree:
   inbag_ind <- which(forest$inbag.counts[[treeind]]!=0)
   data <- data[inbag_ind,]
-  data[-ncol(data)] <- lapply(data[-ncol(data)], as.numeric)
+  data[names(data)!=dep.var.name] <- lapply(data[names(data)!=dep.var.name], as.numeric)
+  
+  # Names of all variables:
+  var.names.all <- names(data)
   
   # Extract the information from the forest for the treeind-th tree:
   scoreval <- forest$score.values[[treeind]]
@@ -468,7 +573,7 @@ reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads =
   
   # Names of categorical variables:
   cat_indicator <- sapply(forest$covariate.levels, length)>0
-  var.names.cat <- var.names.all[cat_indicator]
+  var.names.cat <- independent.variable.names[cat_indicator]
   var.cat.levels <- forest$covariate.levels[cat_indicator]
   
   # This will contain the labels showing the class distributions at the nodes:
@@ -478,12 +583,20 @@ reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads =
   indices_nodes <- vector("list", length = length(nodeID_in_root))
   indices_nodes[[1]] <- 1:nrow(data)
   
+  if (treetype %in% c(1, 9)) {
   # First class distribution label:
   nums_raw <- table(data[indices_nodes[[1]], dep.var.name])
   freqs_raw <- nums_raw/sum(nums_raw)
   
-  ytab <- round(freqs_raw, 2) # CHANGED: 3)
+  ytab <- round(freqs_raw, 2)
   y_label[1] <- paste(paste0(names(ytab), ": ", ytab), collapse=", ")
+  }
+  else {
+  quartiles_raw <- quantile(data[indices_nodes[[1]], dep.var.name], c(0.25, 0.5, 0.75))
+  quartiles <- sapply(quartiles_raw, format_number)
+  
+  y_label[1] <- paste0("Med.: ", quartiles[2], " [0.25 Qu.: ", quartiles[1], ", 0.25 Qu.: ", quartiles[3], "]")
+  }
   
   
   # First line of the data.frame that will contain the horizontal lines showing
@@ -507,10 +620,15 @@ reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads =
   # returned by the function:
   rules_dist <- vector("list", length = length(nodeID_in_root))
   
+  if (treetype %in% c(1, 9)) {
   rules_dist_new <- rbind(freqs_raw, nums_raw)
   rownames(rules_dist_new) <- c("frequencies", "numbers")
   
   rules_dist[[1]] <- rules_dist_new
+  }
+  else {
+    rules_dist[[1]] <- quartiles_raw
+  }
   
   # This will contain the rules corresponding to the nodes in the forest:
   rules <- rep("root node", length(nodeID_in_root))
@@ -536,15 +654,27 @@ reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads =
         indices_nodes[[i]] <- indices_nodes[[ind_parent]][data[indices_nodes[[ind_parent]],var.names.all[split_varIDs[ind_parent]]] > split_values[ind_parent]]
       }
       
-      nums_raw <- table(data[indices_nodes[[i]], dep.var.name])
+      if (treetype %in% c(1, 9)) {
+      nums_raw <- table(data[indices_nodes[[i]], dep.var.name]) #          HIER
       freqs_raw <- nums_raw/sum(nums_raw)
-      ytab <- round(freqs_raw, 2) # CHANGED: 3)
+      ytab <- round(freqs_raw, 2)
       y_label[i] <- paste(paste0(names(ytab), ": ", ytab), collapse=", ")
       
       rules_dist_new <- rbind(freqs_raw, nums_raw)
       rownames(rules_dist_new) <- c("frequencies", "numbers")
       
       rules_dist[[i]] <- rules_dist_new
+      }
+      else {
+        
+        quartiles_raw <- quantile(data[indices_nodes[[i]], dep.var.name], c(0.25, 0.5, 0.75))
+        quartiles <- sapply(quartiles_raw, format_number)
+        
+        y_label[i] <- paste0("Med.: ", quartiles[2], " [0.25 Qu.: ", quartiles[1], ", 0.25 Qu.: ", quartiles[3], "]")
+        
+        rules_dist[[i]] <- quartiles_raw
+        
+      }
       
       parent_line <- xlines_df[xlines_df$ind_in_full_tree==ind_parent,]    
       
@@ -646,12 +776,20 @@ reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads =
   # The widths of the horizontal lines are determined by scores that express whether
   # the corresponding variables are used more often in the best trees associated with
   # the variables than in the complete forest - however, for the variable for which
-  # the representative tree was obtained this does not make sense because this variable
+  # the representative tree was obtained, this does not make sense because this variable
   # is obviously in all best trees that use this variable.
   # Therefore for the variable for which the representative tree was obtained we
   # use as width the mean score value from the score values of the splits in the
   # other variables:
+  if (any(xlines_df$split_var!=var.names[treeind])) {
   xlines_df$scoreval[xlines_df$split_var==var.names[treeind]] <- mean(xlines_df$scoreval[xlines_df$split_var!=var.names[treeind]])
+  }
+  else {
+    # If the tree root only uses the variable for which the representative tree was obtained, 
+    # we simply set the score value to 1 for all splits:
+    xlines_df$scoreval <- 1
+  }
+  
   
   
   
@@ -756,7 +894,7 @@ reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads =
   indices_best_splits <- xlines_df$ind_in_full_tree[xlines_df$is_in_best==1 & xlines_df$split_var==var.names[treeind]]
   
   
-  if (box_plots | density_plots) {
+  if (box_plots | density_plots | scatter_plots) {
     
     for (i in seq(along=indices_best_splits)) {
       
@@ -767,8 +905,6 @@ reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads =
       split_value <- split_values[index_temp]
       split_label <- names(rules_dist)[which(y_label_df$ind_in_full_tree == index_temp)]
       
-      counts <- make_count_labels(data_best_split, dep.var.name, split_var)
-      
       if (split_label=="root node")
         plot_title <- paste0("Marginal influence of ", split_var)
       else
@@ -781,8 +917,12 @@ reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads =
         levelstemp <- var.cat.levels[[which(var.names.cat==split_var)]]
         xtemp <- factor(levelstemp[xtemp], levels=levelstemp)
       }
+
+      if (treetype %in% c(1,9)) {
+
+      counts <- make_count_labels(data_best_split, dep.var.name, split_var)
       
-      if (!inherits(xtemp, "factor") && length(unique(xtemp)) > 2) {
+      if (!inherits(xtemp, "factor") && length(unique(xtemp)) > 4) {
         if (box_plots) {
           p <- plotVarBoxplotUFO(x=xtemp, y=ytemp, counts=counts, split_value=split_value, x_label=dep.var.name, y_label=split_var, plot_title=plot_title, add_split_line=add_split_line)
           ps[[length(ps) + 1]] <- p
@@ -799,6 +939,22 @@ reprTrees <- function(object, vars=NULL, numvars=5, indvars=NULL, num.threads =
         ps[[length(ps) + 1]] <- p
         names(ps)[length(ps)] <- ifelse(length(indices_best_splits) > 1, paste0("bar_plot_", i), "bar_plot")
       }
+	  
+	  }
+	  
+	  if (treetype == 3) {
+	  
+	     if (!inherits(xtemp, "factor") && length(unique(xtemp)) > 4) {
+          p <- plotVarScatterplotUFO(x=xtemp, y=ytemp, split_value=split_value, x_label=split_var, y_label=dep.var.name, plot_title=plot_title, add_split_line=add_split_line)
+          ps[[length(ps) + 1]] <- p
+          names(ps)[length(ps)] <- ifelse(length(indices_best_splits) > 1, paste0("scatter_plot_", i), "scatter_plot")
+		 } else {
+          p <- plotVarBoxplotMetrUFO(x=xtemp, y=ytemp, split_value=split_value, x_label=split_var, y_label=dep.var.name, plot_title=plot_title, add_split_line=add_split_line)
+          ps[[length(ps) + 1]] <- p
+          names(ps)[length(ps)] <- ifelse(length(indices_best_splits) > 1, paste0("scatter_plot_", i), "scatter_plot")
+		 }
+	  
+	  }
       
     }
     
@@ -953,14 +1109,14 @@ plotVarBoxplotUFO <- function(x, y, counts, split_value, x_label="", y_label="",
   # Add labels to the plot if provided:
   
   if (x_label=="")
-    p <- p + ggplot2::theme(axis.title.x=ggplot2::element_blank())
-  else
-    p <- p + ggplot2::ylab(x_label)
-  
-  if (y_label=="")
     p <- p + ggplot2::xlab("class")
   else
-    p <- p + ggplot2::xlab(y_label)
+    p <- p + ggplot2::xlab(x_label)
+  
+  if (y_label=="")
+    p <- p + ggplot2::theme(axis.title.x=ggplot2::element_blank())
+  else
+    p <- p + ggplot2::ylab(y_label)
   
   if (plot_title!="")
     p <- p + ggplot2::ggtitle(plot_title)
@@ -976,6 +1132,7 @@ plotVarDensityUFO <- function(x, y, counts, split_value, x_label="", y_label="",
   
   # The densities are plotted only for classes with at least two observations:
   levels_to_keep <- names(classtab[classtab >= 2])
+  counts <- counts[counts$label != "n = 1",]
   
   filterbool <- y %in% levels_to_keep
   
@@ -1162,9 +1319,10 @@ plotVarBarplotUFO <- function(x, y, split_value, x_label="", legend_title="", pl
     label = paste0("n = ", .data$n)
   )
   counts <- dplyr::ungroup(counts)
+  counts <- dplyr::filter(counts, .data$n != 0)
   
   # Plot
-  p <- ggplot2::ggplot(counts, ggplot2::aes(x = .data$x, y = .data$prop, fill = .data$y)) +
+  p <- ggplot2::ggplot(counts, ggplot2::aes(x = factor(.data$x), y = .data$prop, fill = .data$y)) +
     ggplot2::geom_col() +
     ggplot2::geom_text(
       ggplot2::aes(label = .data$label),
@@ -1184,8 +1342,131 @@ plotVarBarplotUFO <- function(x, y, split_value, x_label="", legend_title="", pl
       legend.text = ggplot2::element_text(size = 15)
     )
   
-  if (add_split_line) 
+  if (add_split_line) {
+    
+    if (is.factor(counts$x)) {
+      # Sometimes not all factor categories are represented in the plot.
+      # In these cases, we have to recalculate the split value because the 
+      # split value is with respect to all categories. But if there are 
+      # categories missing, we have to recompute the split value in order 
+      # to make it appear at the right position. 
+      unique_vals <- sort(unique(as.numeric(counts$x)))
+      split_value <- max(which(unique_vals < split_value)) + 0.5
+    } 
+    else {
+      # We do not only show bar plots for factor variables, but also for variables 
+      # coded as metric values which have a maximum of four distinct values. For these, however, 
+      # the split values are not between 1, 2, 3 and so on, but between the numerical values of the variable. 
+      # Therefore for these covariates, we have to set the split value shown in the figure manually
+      # to 1.5 or 2.5 or 3.5 etc.:
+      split_value <- max(which(sort(unique(counts$x)) < split_value)) + 0.5
+    }
+    
     p <- p + ggplot2::geom_vline(xintercept = split_value, linetype = "dashed", color = "green3", linewidth = 1)
+  }
   
   return(p)
 }
+
+plotVarScatterplotUFO <- function(x, y, split_value, x_label, y_label, plot_title, add_split_line)
+{
+  
+  plotdata <- data.frame(x = x, y = y)
+  
+  # If there are more than 1000 observations, the plot should show only a
+  # random subset of 1000 observations:
+  if (nrow(plotdata) > 1000) {
+    plotdata <- plotdata[sample(1:nrow(plotdata), size=1000),]
+  }
+  
+  p <- ggplot2::ggplot(plotdata, ggplot2::aes(x = .data$x, y = .data$y)) + 
+    ggplot2::theme_bw() + ggplot2::geom_point() +
+    ggplot2::theme(
+      axis.text.x = ggplot2::element_text(color = "black", size = 11),
+      axis.text.y = ggplot2::element_text(color = "black", size = 11),
+      axis.title = ggplot2::element_text(size = 15),
+      plot.title = ggplot2::element_text(size = 15)
+    )
+  
+  if (add_split_line) 
+    p <- p + ggplot2::geom_vline(xintercept = split_value, linetype = "dashed", color = "green3", linewidth = 1)
+  
+  if (x_label=="")
+    p <- p + ggplot2::theme(axis.title.x=ggplot2::element_blank())
+  else
+    p <- p + ggplot2::xlab(x_label)
+  
+  if (y_label!="")
+    p <- p + ggplot2::ylab(y_label)
+  
+  if (plot_title!="")
+    p <- p + ggplot2::ggtitle(plot_title)
+  
+  return(p)
+  
+}
+
+plotVarBoxplotMetrUFO <- function(x, y, split_value, x_label="", y_label="", plot_title="", add_split_line=TRUE) {
+
+  res <- by(y, factor(x), function(vals) {
+    n    <- length(vals)
+    rng  <- range(vals, na.rm = TRUE)
+    ymax <- max(vals, na.rm = TRUE)
+    ypos <- ymax + 0.05 * diff(rng)
+    data.frame(n = n, y = ypos)
+  })
+  
+  counts <- do.call(rbind, res)
+  counts$category <- factor(rownames(counts), levels = levels(factor(x)))
+  counts$label <- paste0("n = ", counts$n)
+  rownames(counts) <- NULL
+
+  plotdata <- data.frame(x=x, y=y)
+  
+  p <- ggplot2::ggplot(plotdata, ggplot2::aes(x = factor(.data$x), y = .data$y)) + ggplot2::theme_bw() + ggplot2::geom_boxplot() +
+    ggplot2::geom_text(data = counts, ggplot2::aes(x = .data$category, y = .data$y, label = .data$label), inherit.aes = FALSE, color = "grey40", size = 5) +
+    ggplot2::theme(
+      axis.text.x = ggplot2::element_text(color = "black", size = 15),
+      axis.text.y = ggplot2::element_text(color = "black", size = 11),
+      axis.title = ggplot2::element_text(size = 15),
+      plot.title = ggplot2::element_text(size = 15)
+    )
+  
+  if (is.factor(plotdata$x)) {
+    # Sometimes not all factor categories are represented in the plot.
+    # In these cases, we have to recalculate the split value because the 
+    # split value is with respect to all categories. But if there are 
+    # categories missing, we have to recompute the split value in order 
+    # to make it appear at the right position. 
+    unique_vals <- sort(unique(as.numeric(plotdata$x)))
+    split_value <- max(which(unique_vals < split_value)) + 0.5
+  } else {
+  # We do not only show boxplots for factor variables, but also for variables 
+  # coded as metric values which have a maximum of four distinct values. For these, however, 
+  # the split values are not between 1, 2, 3 and so on, but between the numerical values of the variable. 
+  # Therefore for these covariates, we have to set the split value shown in the figure manually
+  # to 1.5 or 2.5 or 3.5 etc.:
+  split_value <- max(which(sort(unique(plotdata$x)) < split_value)) + 0.5
+  }
+  
+  if (add_split_line) 
+    p <- p + ggplot2::geom_vline(xintercept = split_value, linetype = "dashed", color = "green3", linewidth = 1)
+  
+  # Add labels to the plot if provided:
+  
+  if (x_label=="")
+    p <- p + ggplot2::theme(axis.title.x=ggplot2::element_blank())
+  else
+    p <- p + ggplot2::xlab(x_label)
+  
+  if (y_label=="")
+    p <- p + ggplot2::ylab("Outcome")
+  else
+    p <- p + ggplot2::ylab(y_label)
+  
+  if (plot_title!="")
+    p <- p + ggplot2::ggtitle(plot_title)
+  
+  p
+  
+}
