@@ -17,11 +17,11 @@
 #  NOTE: 'unityForest' is a fork of the popular R package 'ranger', written by Marvin N. Wright.
 #  Most R and C++ code is identical with that of 'ranger'. The package 'unityForest'
 #  was written by taking the original 'ranger' code and making any
-#  changes necessary to implement diversity forests.
+#  changes necessary to implement unity forests.
 #
 # -------------------------------------------------------------------------------
 
-##' Constructs a unity forest and computes the unity variable importance measure (VIM), as described in Hornung & Hapfelmeier (2026). Currently, only categorical outcomes are supported.\cr
+##' Constructs a unity forest and computes the unity variable importance measure (VIM), as described in Hornung & Hapfelmeier (2026). Categorical and continuous outcomes are supported.\cr
 ##' The unity forest algorithm is a tree construction approach for random forests in which the first few splits are optimized jointly in order to more effectively capture interaction effects beyond marginal effects. The unity VIM quantifies the influence of each variable under the conditions in which that influence is strongest, thereby placing a stronger emphasis on interaction effects than conventional variable importance measures.\cr
 ##' To explore the nature of the effects identified by the unity VIM, it is essential to examine covariate-representative tree roots (CRTRs), which are implemented in \code{\link{reprTrees}}.
 ##' 
@@ -33,17 +33,19 @@
 ##' 
 ##' However, note that results obtained from applications of the unity forest framework to high-dimensional datasets should be interpreted with caution. For high-dimensional data, the curse of dimensionality makes the identification of individual interaction effects challenging and increases the risk of false positives. Moreover, the split points identified in the CRTRs (\code{\link{reprTrees}}) may become less precise as the number of covariates considered per tree root increases.
 ##' 
+##' NOTE: The empirical evaluation of the unity forest framework (including the unity forest algorithm, the unity VIM, and covariate-representative tree roots) in Hornung & Hapfelmeier (2026) focused on categorical outcomes. Its performance for continuous outcomes has not been systematically investigated. Results for continuous outcomes should therefore be interpreted with appropriate caution.
+##' 
 ##' @title Construct a unity forest prediction rule and compute the unity VIM.
 ##' @param formula Object of class \code{formula} or \code{character} describing the model to fit. Interaction terms supported only for numerical variables.
-##' @param dependent.variable.name Name of outcome variable, needed if no formula given.
+##' @param dependent.variable.name Name of the outcome variable, required if no formula is provided. For categorical outcomes, this variable must be coded as a \code{factor}.
 ##' @param data Training data of class \code{data.frame}, \code{matrix}, \code{dgCMatrix} (Matrix) or \code{gwaa.data} (GenABEL).
 ##' @param num.trees Number of trees. Default is 20000.
 ##' @param num.cand.trees Number of random candidate trees to generate for each tree root. Default is 500.
-##' @param probability Grow a probability forest as in Malley et al. (2012). (NOTE: Currently only probability forests are implemented, will be changed in the next version)
+##' @param probability Grow a probability forest as in Malley et al. (2012). Default is \code{TRUE}. For categorical outcomes only.
 ##' @param importance Variable importance mode, either 'unity' (unity VIM) or 'none'.
 ##' @param prop.best.splits Related to the unity VIM. Default value should generally not be modified by the user. When calculating the unity VIM, only the top \code{prop.best.splits} \eqn{\times} 100\% of the splits -- those with the highest split criterion values weighted by node size -- are considered for each variable. The default value is 0.01, meaning that only the top 1\% of splits are used. While small values are recommended, they should not be set too low to ensure that each variable has a sufficient number of splits for a reliable unity VIM computation.
 ##' @param min.node.size.root Minimal node size in the tree roots. Default is 10 irrespective of the outcome type.
-##' @param min.node.size Minimal node size. Default 1 for classification and 5 for probability.
+##' @param min.node.size Minimal node size. Default 5 for probability, 5 for classification, and 5 for continuous outcomes.
 ##' @param max.depth.root Maximal depth of the tree roots. Default value is 3 and should generally not be modified by the user. Larger values can be associated with worse predictive performance for some datasets.
 ##' @param max.depth Maximal tree depth. A value of NULL or 0 (the default) corresponds to unlimited depth, 1 to tree stumps (1 split per tree). Must be at least as large as \code{max.depth.root}.
 ##' @param prop.var.root Proportion of variables randomly sampled for constructing each tree root. Default is the square root of the number of variables divided by the number of variables. Consequently, per default, for each tree root, a random subset of variables is considered, with size equal to the (rounded up) square root of the total number of variables. An exception is made for datasets with more than 100 variables, where the default for \code{prop.var.root} is set to 0.1. See the 'Details' section below for explanation.
@@ -54,17 +56,24 @@
 ##' @param class.weights Weights for the outcome classes (in order of the factor levels) in the splitting rule (cost sensitive learning). Classification and probability prediction only. For classification the weights are also applied in the majority vote in terminal nodes.
 ##' @param inbag Manually set observations per tree. List of size \code{num.trees}, containing inbag counts for each observation. Can be used for stratified sampling.
 ##' @param oob.error Compute OOB prediction error. Set to \code{FALSE} to save computation time.
-##' @param num.threads Number of threads. Default is number of CPUs available.
+##' @param num.threads Number of threads to use.
+##' The default is to use at most 2 threads (and at most the number of available CPU cores).
+##' This conservative default avoids unintentionally using many cores on shared computing resources
+##' (e.g., CI systems, servers, or HPC login/compute nodes).
+##'
+##' For typical use on a personal computer, setting \code{num.threads = 0} is strongly recommended,
+##' as it uses all available CPU cores, which typically substantially reduces runtime.
 ##' @param write.forest Save \code{unityfor.forest} object, required for prediction. Set to \code{FALSE} to reduce memory usage if no prediction intended.
 ##' @param verbose Show computation status and estimated runtime.
 ##' @return Object of class \code{unityfor} with elements
-##'   \item{\code{predictions}}{Predicted classes/values, based on out-of-bag samples.}
+##'   \item{\code{predictions}}{Predicted classes/class probabilities/values, based on out-of-bag samples.}
 ##'   \item{\code{forest}}{Saved forest (If write.forest set to TRUE). Note that the variable IDs in the \code{split.varIDs} object do not necessarily represent the column number in R.}
 ##'   \item{\code{data}}{Training data.}
 ##'   \item{\code{variable.importance}}{Variable importance for each independent variable. Only available if \code{importance} is not \code{"none"}.}
 ##'   \item{\code{importance.mode}}{Importance mode used.}
-##'   \item{\code{prediction.error}}{Overall out-of-bag prediction error. For classification this is the fraction of missclassified samples, for probability estimation the Brier score and for regression the mean squared error.}
+##'   \item{\code{prediction.error}}{Overall out-of-bag prediction error. For classification this is the fraction of missclassified samples, for probability estimation the Brier score and for continuous outcomes the mean squared error.}
 ##'   \item{\code{confusion.matrix}}{Contingency table for classes and predictions based on out-of-bag samples (classification only).}
+##'   \item{\code{r.squared}}{R squared. Also called explained variance or coefficient of determination (continuous outcomes only). Computed on out-of-bag data.}
 ##'   \item{\code{call}}{Function call.}
 ##'   \item{\code{num.trees}}{Number of trees.}
 ##'   \item{\code{num.cand.trees}}{Number of candidate trees generated for each tree root.}
@@ -77,8 +86,19 @@
 ##'   \item{\code{min.node.size}}{Value of minimal node size used.}
 ##'   \item{\code{splitrule}}{Splitting rule (used only in the tree sprouts).}
 ##'   \item{\code{replace}}{Sample with replacement.}
-##'   \item{\code{treetype}}{Type of forest/tree. Classification or regression.}
+##'   \item{\code{treetype}}{Type of forest/tree. Categorical or continous outcome.}
 ##' @examples
+##' 
+##' ## IMPORTANT NOTE on parallelization:
+##' ## The default uses at most 2 threads (num.threads) to avoid unintentionally
+##' ## using many cores on shared systems.
+##' ## However, for typical runs on a personal computer, set num.threads = 0 to 
+##' ## use all available CPU cores; this is strongly recommended and can 
+##' ## substantially reduce runtime.
+##' ## Note: num.threads = 1 is used in the examples to avoid parallel
+##' ## execution during package checks.
+##' 
+##' 
 ##' ## Load package:
 ##' 
 ##' library("unityForest")
@@ -97,7 +117,7 @@
 ##' ## Construct unity forest and calculate unity VIM values:
 ##' 
 ##' model <- unityfor(dependent.variable.name = "C", data = wine,
-##'                   importance = "unity", num.trees = 20)
+##'                   importance = "unity", num.trees = 20, num.threads = 1)
 ##' 
 ##' # NOTE: num.trees = 20 (in the above) would be much too small for practical 
 ##' # purposes. This small number of trees was simply used to keep the
@@ -122,17 +142,49 @@
 ##' # Construct unity forest on training data:
 ##' # NOTE again: num.trees = 20 is specified too small for practical purposes.
 ##' model_train <- unityfor(dependent.variable.name = "C", data = wine_train, 
-##'                         importance = "none", num.trees = 20)
+##'                         importance = "none", num.trees = 20, 
+##'                         probability = FALSE, num.threads = 1)
+##' # NOTE: Because we are only interested in prediction here, we do not
+##' # calculate unity VIM values (by setting importance = "none"), because 
+##' # this speeds up calculations.
+##' # Moreover, 'probability' is set to 'FALSE' because we are interested in pure
+##' # class prediction in this example (without class probability prediction).
+##' 
+##' # Predict class values of the test data:
+##' pred_wine <- predict(model_train, data = wine_test, num.threads = 1)
+##' 
+##' # Compare predicted and true class values of the test data:
+##' table(wine_test$C, pred_wine$predictions)
+##' 
+##' 
+##' 
+##' ## Prediction for dataset with continuous outcome:
+##' 
+##' # Load stock dataset:
+##' 
+##' data(stock)
+##' 
+##' # Separate 'stock' dataset randomly in training
+##' # and test data:
+##' train.idx <- sample(nrow(stock), 2/3 * nrow(stock))
+##' stock_train <- stock[train.idx, ]
+##' stock_test <- stock[-train.idx, ]
+##' 
+##' # Construct unity forest on training data:
+##' # NOTE again: num.trees = 20 is specified too small for practical purposes.
+##' model_train <- unityfor(dependent.variable.name = "company10", 
+##'                         data = stock_train, importance = "none", 
+##'                         num.trees = 20, num.threads = 1)
 ##' # NOTE: Because we are only interested in prediction here, we do not
 ##' # calculate unity VIM values (by setting importance = "none"), because 
 ##' # this speeds up calculations.
 ##' 
-##' # Predict class values of the test data:
-##' pred_wine <- predict(model_train, data = wine_test)
+##' # Predict outcome values of the test data:
+##' pred_stock <- predict(model_train, data = stock_test, num.threads = 1)
+##' 
+##' # Compare predicted and true outcome values of the test data:
+##' plot(pred_stock$predictions, stock_test$company10)
 ##' 
-##' # Compare predicted and true class values of the test data:
-##' table(wine_test$C, levels(wine_train$C)[apply(pred_wine$predictions, 1, which.max)])
-##'
 ##' @author Roman Hornung, Marvin N. Wright
 ##' @references
 ##' \itemize{
@@ -159,12 +211,6 @@ unityfor <- function(formula = NULL, dependent.variable.name = NULL, data = NULL
 					 
   classification <- NULL
   
-  ## Currently only probability forests are implemented:
-  if (!probability) {
-    probability <- TRUE
-    warning("Currently only probability forests are implemented. Therefore, probability was set to TRUE")
-  }
-    
   ## For unity forests we always order the categories of categorical variables:
   respect.unordered.factors <- "order"
   save.memory <- FALSE
@@ -245,11 +291,16 @@ unityfor <- function(formula = NULL, dependent.variable.name = NULL, data = NULL
   } else if (is.numeric(response) && (is.null(ncol(response)) || ncol(response) == 1)) {
     if (!is.null(classification) && classification && !probability) {
       treetype <- 1
-    } else if (probability) {
-      treetype <- 9
     } else {
       treetype <- 3
     }
+    #if (!is.null(classification) && classification && !probability) {
+    #  treetype <- 1
+    #} else if (probability) {
+    #  treetype <- 9
+    #} else {
+    #  treetype <- 3
+    #}
   } else if (inherits(response, "Surv") || is.data.frame(response) || is.matrix(response)) {
     treetype <- 5
   } else {
@@ -352,13 +403,29 @@ unityfor <- function(formula = NULL, dependent.variable.name = NULL, data = NULL
   }
     
   ## Num threads
-  ## Default 0 -> detect from system in C++.
+  ## Default: use at most 2 threads. Set num.threads = 0 to use all available cores.
   if (is.null(num.threads)) {
-    num.threads = 0
+    num.threads <- 2L
   } else if (!is.numeric(num.threads) || num.threads < 0) {
     stop("Error: Invalid value for num.threads")
   }
   
+  ## Respect core limits during R CMD check / CRAN-like checks
+  ## (only if running under package checks)
+  if (nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_", unset = ""))) {
+    limit_cores <- Sys.getenv("_R_CHECK_LIMIT_CORES_", unset = "")
+    if (nzchar(limit_cores)) {
+      lim <- suppressWarnings(as.integer(limit_cores))
+      if (!is.na(lim) && lim > 0) {
+        if (num.threads == 0) {
+          num.threads <- lim
+        } else {
+          num.threads <- min(num.threads, lim)
+        }
+      }
+    }
+  }
+  
   ## Minumum node size
   if (is.null(min.node.size)) {
     min.node.size <- 0
@@ -536,8 +603,8 @@ unityfor <- function(formula = NULL, dependent.variable.name = NULL, data = NULL
                       save.memory, splitrule.num, case.weights, use.case.weights, class.weights, 
                       predict.all, keep.inbag, sample.fraction, alpha=0.5, minprop=0.1, holdout = FALSE, prediction.type, 
                       num_random_splits=1, sparse.data, use.sparse.data, order.snps, oob.error, max.depth, max.depth.root, num.cand.trees,
-                      inbag, use.inbag, nsplits=mtry.sprout, npairs=0, proptry=0, prop.var.root, divfortype=4, promispairs=list(0,0), eim_mode=0, 
-                      metricind=numeric(0), prop.best.splits, repr.tree.mode, repr.var.names)
+                      inbag, use.inbag, prop.var.root,
+                      prop.best.splits, repr.tree.mode, repr.var.names)
   
   if (length(result) == 0) {
     stop("User interrupt or internal error.")
@@ -608,10 +675,7 @@ unityfor <- function(formula = NULL, dependent.variable.name = NULL, data = NULL
   result$num.cand.trees <- num.cand.trees
   result$max.depth.root <- max.depth.root
   result$min.node.size.root <- min.node.size.root
-  
-  result$nsplits <- NULL
-  result$proptry <- NULL
-  result$npairs <- NULL
+
   result$data <- data.selected
   result$prop.best.splits <- prop.best.splits
   result$prop.var.root <- prop.var.root
@@ -620,7 +684,7 @@ unityfor <- function(formula = NULL, dependent.variable.name = NULL, data = NULL
   # Reorder the elements of the "result" list so that they have a more meaningful order:
   
   res_names_all <- c("forest", "predictions", "data", "variable.importance", "importance.mode", "prop.best.splits", "prediction.error",
-                     "confusion.matrix", "call", "num.trees", "num.cand.trees",
+                     "confusion.matrix", "r.squared", "call", "num.trees", "num.cand.trees",
                      "num.independent.variables", "num.samples", "prop.var.root", "mtry",
                      "max.depth.root", "min.node.size.root", "min.node.size", "splitrule", "replace",
                      "treetype")
