@@ -2,6 +2,7 @@
  This file is part of unityForest.
 
  Copyright (c) [2014-2018] [Marvin N. Wright]
+ Modifications and extensions by Roman Hornung
 
  This software may be modified and distributed under the terms of the MIT license.
 
@@ -15,22 +16,26 @@
 
 #include "DataFloat.h"
 
-namespace unityForest {
+namespace unityForest
+{
 
-DataFloat::DataFloat(double* data_double, std::vector<std::string> variable_names, size_t num_rows, size_t num_cols) {
-  this->variable_names = variable_names;
-  this->num_rows = num_rows;
-  this->num_cols = num_cols;
-  this->num_cols_no_snp = num_cols;
+  DataFloat::DataFloat(double *data_double, std::vector<std::string> variable_names, size_t num_rows, size_t num_cols)
+  {
+    this->variable_names = variable_names;
+    this->num_rows = num_rows;
+    this->num_cols = num_cols;
+    this->num_cols_no_snp = num_cols;
 
-  reserveMemory();
-  for (size_t i = 0; i < num_cols; ++i) {
-    for (size_t j = 0; j < num_rows; ++j) {
-      data[i * num_rows + j] = (float) data_double[i * num_rows + j];
+    reserveMemory();
+    for (size_t i = 0; i < num_cols; ++i)
+    {
+      for (size_t j = 0; j < num_rows; ++j)
+      {
+        data[i * num_rows + j] = (float)data_double[i * num_rows + j];
+      }
     }
   }
-}
 
-// #nocov end
+  // #nocov end
 
-}// namespace unityForest
+} // namespace unityForest
