@@ -14,30 +14,31 @@
  You should have received a copy of the GNU General Public License
  along with unityForest. If not, see <http://www.gnu.org/licenses/>.
 
- Written by:
+ This file is based on the ranger package.
 
- Marvin N. Wright
- Institut für Medizinische Biometrie und Statistik
- Universität zu Lübeck
- Ratzeburger Allee 160
- 23562 Lübeck
- Germany
+ Original author:
+   Marvin N. Wright
+   Institut für Medizinische Biometrie und Statistik
+   Universität zu Lübeck
+   http://www.imbs-luebeck.de
 
- http://www.imbs-luebeck.de
+ Modifications and extensions for the unityForest package:
+   Roman Hornung
  #-------------------------------------------------------------------------------*/
 
 #include "DataSparse.h"
 
-namespace unityForest {
+namespace unityForest
+{
 
-DataSparse::DataSparse(Eigen::SparseMatrix<double>& data, std::vector<std::string> variable_names, size_t num_rows,
-    size_t num_cols) :
-    data { } {
-  this->data.swap(data);
-  this->variable_names = variable_names;
-  this->num_rows = num_rows;
-  this->num_cols = num_cols;
-  this->num_cols_no_snp = num_cols;
-}
+  DataSparse::DataSparse(Eigen::SparseMatrix<double> &data, std::vector<std::string> variable_names, size_t num_rows,
+                         size_t num_cols) : data{}
+  {
+    this->data.swap(data);
+    this->variable_names = variable_names;
+    this->num_rows = num_rows;
+    this->num_cols = num_cols;
+    this->num_cols_no_snp = num_cols;
+  }
 
 } // namespace unityForest
