@@ -14,15 +14,16 @@
  You should have received a copy of the GNU General Public License
  along with unityForest. If not, see <http://www.gnu.org/licenses/>.
 
- Written by:
+ This file is based on the ranger package.
 
- Marvin N. Wright
- Institut für Medizinische Biometrie und Statistik
- Universität zu Lübeck
- Ratzeburger Allee 160
- 23562 Lübeck
+ Original author:
+   Marvin N. Wright
+   Institut für Medizinische Biometrie und Statistik
+   Universität zu Lübeck
+   http://www.imbs-luebeck.de
 
- http://www.imbs-luebeck.de
+ Modifications and extensions for the unityForest package:
+   Roman Hornung
  #-------------------------------------------------------------------------------*/
 
 #include <RcppEigen.h>
@@ -43,6 +44,8 @@
 #include "DataSparse.h"
 #include "utility.h"
 
+//#include "debug_cp.h"
+
 using namespace unityForest;
 
 // [[Rcpp::depends(RcppEigen)]]
@@ -58,16 +61,17 @@ Rcpp::List divforCpp(uint treetype, std::string dependent_variable_name, Rcpp::N
                      bool use_case_weights, std::vector<double> &class_weights, bool predict_all, bool keep_inbag,
                      std::vector<double> &sample_fraction, double alpha, double minprop, bool holdout, uint prediction_type_r,
                      uint num_random_splits, Eigen::SparseMatrix<double> &sparse_data, bool use_sparse_data, bool order_snps,
-                     bool oob_error, uint max_depth, uint max_depth_root, uint num_cand_trees, std::vector<std::vector<size_t>> &inbag, bool use_inbag, uint nsplits, uint npairs, double proptry, 
-					 double prop_var_root, uint divfortype, std::vector<std::vector<size_t>> &promispairs, uint eim_mode,
-					 std::vector<size_t> &metricind, double prop_best_splits, bool repr_tree_mode, std::vector<std::string> repr_var_names)
+                     bool oob_error, uint max_depth, uint max_depth_root, uint num_cand_trees, std::vector<std::vector<size_t>> &inbag, bool use_inbag,
+                     double prop_var_root, double prop_best_splits, bool repr_tree_mode, std::vector<std::string> repr_var_names)
 {
 
   Rcpp::List result;
 
   try
   {
-	  
+
+    // CP();
+
     std::unique_ptr<Forest> forest{};
     std::unique_ptr<Data> data{};
 
@@ -164,93 +168,81 @@ Rcpp::List divforCpp(uint treetype, std::string dependent_variable_name, Rcpp::N
     SplitRule splitrule = (SplitRule)splitrule_r;
     PredictionType prediction_type = (PredictionType)prediction_type_r;
 
-    // Init divfor
+    // Init unityfor
     forest->initR(dependent_variable_name, std::move(data), mtry, num_trees, verbose_out, seed, num_threads,
                   importance_mode, min_node_size, min_node_size_root, split_select_weights, always_split_variable_names, status_variable_name,
                   prediction_mode, sample_with_replacement, unordered_variable_names, save_memory, splitrule, case_weights,
-                  inbag, predict_all, keep_inbag, sample_fraction, nsplits, npairs, proptry, prop_var_root, alpha, minprop, holdout, prediction_type, num_random_splits,
-                  order_snps, max_depth, max_depth_root, num_cand_trees, promispairs, eim_mode, divfortype, metricind, prop_best_splits, repr_tree_mode, repr_var_names);
+                  inbag, predict_all, keep_inbag, sample_fraction, prop_var_root, alpha, minprop, holdout, prediction_type, num_random_splits,
+                  order_snps, max_depth, max_depth_root, num_cand_trees, prop_best_splits, repr_tree_mode, repr_var_names);
 
-    // Load forest object if in prediction mode
+    // CP();
+    //  Load forest object if in prediction mode
     if (prediction_mode)
     {
-		
+
       size_t dependent_varID = loaded_forest["dependent.varID"];
       // size_t num_trees = loaded_forest["num.trees"];
       std::vector<std::vector<std::vector<size_t>>> child_nodeIDs = loaded_forest["child.nodeIDs"];
       std::vector<std::vector<size_t>> split_varIDs = loaded_forest["split.varIDs"];
       std::vector<std::vector<double>> split_values = loaded_forest["split.values"];
-      std::vector<std::vector<size_t>> split_types = loaded_forest["split.types"];
-      std::vector<std::vector<std::vector<size_t>>> split_multvarIDs = loaded_forest["split.multvarIDs"];
-      std::vector<std::vector<std::vector<std::vector<bool>>>> split_directs = loaded_forest["split.directs"];
-      std::vector<std::vector<std::vector<std::vector<double>>>> split_multvalues = loaded_forest["split.multvalues"];
       std::vector<bool> is_ordered = loaded_forest["is.ordered"];
 
-      if (!repr_tree_mode) {
-      if (treetype == TREE_CLASSIFICATION)
+      if (!repr_tree_mode)
       {
-        std::vector<std::vector<std::vector<size_t>>> child_muwnodeIDs = loaded_forest["child.muwnodeIDs"];
-        std::vector<std::vector<std::vector<double>>> split_muwvalues = loaded_forest["split.muwvalues"];
-        std::vector<double> class_values = loaded_forest["class.values"];
-        auto &temp = dynamic_cast<ForestClassification &>(*forest);
-        temp.loadForest(dependent_varID, num_trees, child_nodeIDs, split_varIDs, split_values, split_types, split_multvarIDs,
-                        split_directs, split_multvalues, child_muwnodeIDs, split_muwvalues, class_values, is_ordered);
+        if (treetype == TREE_CLASSIFICATION)
+        {
+          std::vector<double> class_values = loaded_forest["class.values"];
+          auto &temp = dynamic_cast<ForestClassification &>(*forest);
+          temp.loadForest(dependent_varID, num_trees, child_nodeIDs, split_varIDs, split_values,
+                          class_values, is_ordered);
+        }
+        else if (treetype == TREE_REGRESSION)
+        {
+          auto &temp = dynamic_cast<ForestRegression &>(*forest);
+          temp.loadForest(dependent_varID, num_trees, child_nodeIDs, split_varIDs, split_values, is_ordered);
+        }
+        else if (treetype == TREE_PROBABILITY)
+        {
+          std::vector<double> class_values = loaded_forest["class.values"];
+          std::vector<std::vector<std::vector<double>>> terminal_class_counts = loaded_forest["terminal.class.counts"];
+          auto &temp = dynamic_cast<ForestProbability &>(*forest);
+          temp.loadForest(dependent_varID, num_trees, child_nodeIDs, split_varIDs, split_values,
+                          class_values,
+                          terminal_class_counts, is_ordered);
+        }
       }
-      else if (treetype == TREE_REGRESSION)
+      else
       {
-        auto &temp = dynamic_cast<ForestRegression &>(*forest);
-        temp.loadForest(dependent_varID, num_trees, child_nodeIDs, split_varIDs, split_values, split_types, split_multvarIDs,
-                        split_directs, split_multvalues, is_ordered);
-      }
-      else if (treetype == TREE_PROBABILITY)
-      {
-        std::vector<std::vector<std::vector<size_t>>> child_muwnodeIDs = loaded_forest["child.muwnodeIDs"];
-        std::vector<std::vector<std::vector<double>>> split_muwvalues = loaded_forest["split.muwvalues"];
-        std::vector<double> class_values = loaded_forest["class.values"];
-        std::vector<std::vector<std::vector<double>>> terminal_class_counts = loaded_forest["terminal.class.counts"];
-        auto &temp = dynamic_cast<ForestProbability &>(*forest);
-        temp.loadForest(dependent_varID, num_trees, child_nodeIDs, split_varIDs, split_values, split_types, split_multvarIDs,
-                        split_directs, split_multvalues, child_muwnodeIDs, split_muwvalues, class_values,
-                        terminal_class_counts, is_ordered);
-      }
-	  } else {
-		  
-		std::vector<std::vector<size_t>> nodeID_in_root = loaded_forest["nodeID.in.root"];
-	    std::vector<std::vector<size_t>> inbag_counts = loaded_forest["inbag.counts"];
 
-		  
-		        if (treetype == TREE_CLASSIFICATION)
-      {
-		//To Do (so wie Tree_probability unten:
-        std::vector<std::vector<std::vector<size_t>>> child_muwnodeIDs = loaded_forest["child.muwnodeIDs"];
-        std::vector<std::vector<std::vector<double>>> split_muwvalues = loaded_forest["split.muwvalues"];
-        std::vector<double> class_values = loaded_forest["class.values"];
-        auto &temp = dynamic_cast<ForestClassification &>(*forest);
-        temp.loadForest(dependent_varID, num_trees, child_nodeIDs, split_varIDs, split_values, split_types, split_multvarIDs,
-                        split_directs, split_multvalues, child_muwnodeIDs, split_muwvalues, class_values, is_ordered);
+        std::vector<std::vector<size_t>> nodeID_in_root = loaded_forest["nodeID.in.root"];
+        std::vector<std::vector<size_t>> inbag_counts = loaded_forest["inbag.counts"];
+
+        if (treetype == TREE_CLASSIFICATION)
+        {
+          std::vector<double> class_values = loaded_forest["class.values"];
+          std::vector<double> class_weights = loaded_forest["class.weights"];
+          auto &temp = dynamic_cast<ForestClassification &>(*forest);
+          temp.loadForestRepr(dependent_varID, num_trees, child_nodeIDs, split_varIDs, split_values,
+                              class_values, class_weights,
+                              nodeID_in_root, inbag_counts, is_ordered);
+        }
+        else if (treetype == TREE_REGRESSION)
+        {
+          auto &temp = dynamic_cast<ForestRegression &>(*forest);
+          temp.loadForestRepr(dependent_varID, num_trees, child_nodeIDs, split_varIDs, split_values,
+                              nodeID_in_root, inbag_counts, is_ordered);
+        }
+        else if (treetype == TREE_PROBABILITY)
+        {
+          std::vector<double> class_values = loaded_forest["class.values"];
+          std::vector<double> class_weights = loaded_forest["class.weights"];
+          std::vector<std::vector<std::vector<double>>> terminal_class_counts = loaded_forest["terminal.class.counts"];
+          auto &temp = dynamic_cast<ForestProbability &>(*forest);
+          temp.loadForestRepr(dependent_varID, num_trees, child_nodeIDs, split_varIDs, split_values,
+                              class_values, class_weights,
+                              terminal_class_counts, nodeID_in_root, inbag_counts, is_ordered);
+        }
       }
-      else if (treetype == TREE_REGRESSION)
-      {
-		//To Do:   
-        auto &temp = dynamic_cast<ForestRegression &>(*forest);
-        temp.loadForest(dependent_varID, num_trees, child_nodeIDs, split_varIDs, split_values, split_types, split_multvarIDs,
-                        split_directs, split_multvalues, is_ordered);
-      }
-      else if (treetype == TREE_PROBABILITY)
-      {
-        std::vector<std::vector<std::vector<size_t>>> child_muwnodeIDs = loaded_forest["child.muwnodeIDs"];
-        std::vector<std::vector<std::vector<double>>> split_muwvalues = loaded_forest["split.muwvalues"];
-        std::vector<double> class_values = loaded_forest["class.values"];
-		std::vector<double> class_weights = loaded_forest["class.weights"];
-        std::vector<std::vector<std::vector<double>>> terminal_class_counts = loaded_forest["terminal.class.counts"];
-        auto &temp = dynamic_cast<ForestProbability &>(*forest);
-        temp.loadForestRepr(dependent_varID, num_trees, child_nodeIDs, split_varIDs, split_values, split_types, split_multvarIDs,
-                        split_directs, split_multvalues, child_muwnodeIDs, split_muwvalues, class_values, class_weights,
-                        terminal_class_counts, nodeID_in_root, inbag_counts, is_ordered);
-      }
-  
-	  }
-	 
     }
     else
     {
@@ -267,12 +259,16 @@ Rcpp::List divforCpp(uint treetype, std::string dependent_variable_name, Rcpp::N
       }
     }
 
+    // CP();
+
     // Print line number:
     // // Rcpp::Rcout << "Line number: " << __LINE__ << std::endl << std::flush;
 
     // Run divfor
     forest->run(false, oob_error);
 
+    // CP();
+
     if (use_split_select_weights && importance_mode != IMP_NONE)
     {
       if (verbose_out)
@@ -282,7 +278,7 @@ Rcpp::List divforCpp(uint treetype, std::string dependent_variable_name, Rcpp::N
             << std::endl;
       }
     }
-	
+
     // Use first non-empty dimension of predictions
     const std::vector<std::vector<std::vector<double>>> &predictions = forest->getPredictions();
     if (predictions.size() == 1)
@@ -308,74 +304,26 @@ Rcpp::List divforCpp(uint treetype, std::string dependent_variable_name, Rcpp::N
     {
       result.push_back(forest->getMtry(), "mtry");
       result.push_back(forest->getMinNodeSize(), "min.node.size");
-      result.push_back(forest->getNsplits(), "nsplits");
-      result.push_back(forest->getNpairs(), "npairs");
-      result.push_back(forest->getProptry(), "proptry");
-      if (divfortype == 1)
+
+      if (importance_mode != IMP_NONE)
       {
-        if (importance_mode != IMP_NONE)
-        {
-          result.push_back(forest->getVariableImportance(), "variable.importance");
-        }
-      }
-      if (divfortype == 3)
-      {
-        if (importance_mode == MUWIMP_MULTIWAY || importance_mode == MUWIMP_BOTH)
-        {
-          if (treetype == TREE_CLASSIFICATION)
-          {
-            auto &temp = dynamic_cast<ForestClassification &>(*forest);
-            result.push_back(temp.getVariableImportanceMuwMultiway(), "var.imp.multiclass");
-          }
-
-          if (treetype == TREE_PROBABILITY)
-          {
-            auto &temp = dynamic_cast<ForestProbability &>(*forest);
-            result.push_back(temp.getVariableImportanceMuwMultiway(), "var.imp.multiclass");
-          }
-
-        }
-
-        if (importance_mode == MUWIMP_DISCR || importance_mode == MUWIMP_BOTH)
-        {
-          if (treetype == TREE_CLASSIFICATION)
-          {
-            auto &temp = dynamic_cast<ForestClassification &>(*forest);
-            result.push_back(temp.getVariableImportanceMuwDiscr(), "var.imp.discr");
-          }
-
-          if (treetype == TREE_PROBABILITY)
-          {
-            auto &temp = dynamic_cast<ForestProbability &>(*forest);
-            result.push_back(temp.getVariableImportanceMuwDiscr(), "var.imp.discr");
-          }
-        }
-
-      }
-	  if (divfortype == 4)
-      {
-        if (importance_mode != IMP_NONE)
-        {
-          result.push_back(forest->getVariableImportance(), "variable.importance");
-        }
+        result.push_back(forest->getVariableImportance(), "variable.importance");
       }
       result.push_back(forest->getOverallPredictionError(), "prediction.error");
     }
 
+    // CP();
+
     // Save forest if needed
     if (write_forest)
     {
-		
+
       Rcpp::List forest_object;
       forest_object.push_back(forest->getDependentVarId(), "dependent.varID");
       forest_object.push_back(forest->getNumTrees(), "num.trees");
       forest_object.push_back(forest->getChildNodeIDs(), "child.nodeIDs");
       forest_object.push_back(forest->getSplitVarIDs(), "split.varIDs");
       forest_object.push_back(forest->getSplitValues(), "split.values");
-      forest_object.push_back(forest->getSplitTypes(), "split.types");
-      forest_object.push_back(forest->getSplitMultVarIDs(), "split.multvarIDs");
-      forest_object.push_back(forest->getSplitDirects(), "split.directs");
-      forest_object.push_back(forest->getSplitMultValues(), "split.multvalues");
       forest_object.push_back(forest->getIsOrderedVariable(), "is.ordered");
 
       if (snp_data.nrow() > 1 && order_snps)
@@ -388,39 +336,35 @@ Rcpp::List divforCpp(uint treetype, std::string dependent_variable_name, Rcpp::N
       if (treetype == TREE_CLASSIFICATION)
       {
         auto &temp = dynamic_cast<ForestClassification &>(*forest);
-        forest_object.push_back(temp.getSplitMuwValues(), "split.muwvalues");
-        forest_object.push_back(temp.getMuwInds(), "muw.inds");
         forest_object.push_back(temp.getClassValues(), "class.values");
-        forest_object.push_back(temp.getChildMuwNodeIDs(), "child.muwnodeIDs");
+        forest_object.push_back(temp.getClassWeights(), "class.weights");
       }
       else if (treetype == TREE_PROBABILITY)
       {
         auto &temp = dynamic_cast<ForestProbability &>(*forest);
-        forest_object.push_back(temp.getSplitMuwValues(), "split.muwvalues");
-        forest_object.push_back(temp.getMuwInds(), "muw.inds");
         forest_object.push_back(temp.getClassValues(), "class.values");
-	    forest_object.push_back(temp.getClassWeights(), "class.weights");
+        forest_object.push_back(temp.getClassWeights(), "class.weights");
         forest_object.push_back(temp.getTerminalClassCounts(), "terminal.class.counts");
-        forest_object.push_back(temp.getChildMuwNodeIDs(), "child.muwnodeIDs");
       }
-	  
-	  forest_object.push_back(forest->getIsInBest(), "is.in.best");
-	  forest_object.push_back(forest->getNodeIDInRoot(), "nodeID.in.root");
+
+      forest_object.push_back(forest->getIsInBest(), "is.in.best");
+      forest_object.push_back(forest->getNodeIDInRoot(), "nodeID.in.root");
 
       forest_object.push_back(forest->getInbagCounts(), "inbag.counts");
 
       result.push_back(forest_object, "forest");
     }
 
-    if (repr_tree_mode) {
-		
+    if (repr_tree_mode)
+    {
+
       Rcpp::List forest_object;
       forest_object.push_back(forest->getDependentVarId(), "dependent.varID");
       forest_object.push_back(forest->getChildNodeIDs(), "child.nodeIDs");
       forest_object.push_back(forest->getSplitVarIDs(), "split.varIDs");
       forest_object.push_back(forest->getSplitValues(), "split.values");
-	  forest_object.push_back(forest->getScoreValues(), "score.values");
-	  forest_object.push_back(forest->getIsOrderedVariable(), "is.ordered");
+      forest_object.push_back(forest->getScoreValues(), "score.values");
+      forest_object.push_back(forest->getIsOrderedVariable(), "is.ordered");
 
       if (snp_data.nrow() > 1 && order_snps)
       {
@@ -440,14 +384,13 @@ Rcpp::List divforCpp(uint treetype, std::string dependent_variable_name, Rcpp::N
         forest_object.push_back(temp.getClassValues(), "class.values");
         forest_object.push_back(temp.getTerminalClassCounts(), "terminal.class.counts");
       }
-	  
-	  forest_object.push_back(forest->getIsInBest(), "is.in.best");
-	  forest_object.push_back(forest->getNodeIDInRoot(), "nodeID.in.root");
-	  forest_object.push_back(forest->getInbagCounts(), "inbag.counts");
+
+      forest_object.push_back(forest->getIsInBest(), "is.in.best");
+      forest_object.push_back(forest->getNodeIDInRoot(), "nodeID.in.root");
+      forest_object.push_back(forest->getInbagCounts(), "inbag.counts");
 
       result.push_back(forest_object, "forest");
-	  
-    }		
+    }
 
     if (!verbose)
     {
@@ -463,6 +406,7 @@ Rcpp::List divforCpp(uint treetype, std::string dependent_variable_name, Rcpp::N
     return result;
   }
 
+  // CP();
+
   return result;
-  
 }
