@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------------
  This file is part of unityForest.
- 
+
  unityForest is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
@@ -14,28 +14,27 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with unityForest. If not, see <http://www.gnu.org/licenses/>.
 
-Written by:
+ This file is based on the ranger package.
 
-Marvin N. Wright
-Institut für Medizinische Biometrie und Statistik
-Universität zu Lübeck
-Ratzeburger Allee 160
-23562 Lübeck
-Germany
+ Original author:
+   Marvin N. Wright
+   Institut für Medizinische Biometrie und Statistik
+   Universität zu Lübeck
+   http://www.imbs-luebeck.de
 
-http://www.imbs-luebeck.de
+ Modifications and extensions for the unityForest package:
+   Roman Hornung
 #-------------------------------------------------------------------------------*/
 
 #include <Rcpp.h>
 
 // Count number of elements in reference smaller than values
 //[[Rcpp::export]]
-Rcpp::IntegerVector numSmaller(Rcpp::NumericVector values, Rcpp::NumericVector reference) {
+Rcpp::IntegerVector numSmaller(Rcpp::NumericVector values, Rcpp::NumericVector reference)
+{
   std::sort(reference.begin(), reference.end());
   Rcpp::IntegerVector result(values.size());
   for (int i = 0; i < values.size(); ++i)
     result[i] = std::lower_bound(reference.begin(), reference.end(), values[i]) - reference.begin();
   return result;
 }
-
-
